## pt2d (Path Tracing in 2D images)

_This repository is no longer being updated as of January 23rd 2025._

### Description

An interactive GUI for path tracing and extraction in 2D images. The inteded use is for 2D medical images of Amphisbaena sutures, but we have found that it extends well into other domains aswell for path extraction in 2D images.

The user places a start and end point between which the program automatically finds a viable path. The user can place multiple new guide points to help fit the path to their liking. The smoothness of the path can be adjusted aswell. Advanced settings are available.

The points making up the final path can be exported to a `.csv` file.

Below is a short demonstration of the basic usage of the GUI.

![General usage of the GUI](./images/GUI_gif.gif)

### Installation

The program can be installed using pip:

```plaintext
pip install pt2d
```

### Usage

To open the program type out `pt2d` in a terminal. The GUI looks the following:

![General GUI layout](./images/GUI.PNG)

#### General usage
The user can add in an image by pressing `Load Image`. 
After loading in an image, two anchor points are loaded in intially: start (S) and end (E). The user places these at the start and end point of the path they wish to extract by dragging them.
The user can add more anchor points by left-clicking anywhere on the image. Anchor points (except from start and end) can be removed by right-clicking on them. The user can also press `Clear Points` to remove all at once.

The program will try to find the path live as points are placed and dragged.
Once the desired path has been found it can be extracted by pressing `Export Path`.

#### Advanced usage
The GUI has a set of advanced settings that can be acessed by pressing `Advanced Settings`. These include:
* By pressing `Toggle Rainbow`, the path is colored with a gradient that transitions smoothly through a spectrum of colors, creating a rainbow effect. This allows the user to verify that the path is in correct order.
* The smoothness of the path can be adjusted using the `Line smoothing` slider. A higher value results in a smoother line.
* The applied contrasting can be adjusted using the `Contrast` slider. A higher value results in more contrast in the image. This directly affects the contrasted blurred image and the cost image, which are also displayed in the advanced settings.
* The radius of the disk kernel convolved with the contrasted blurred image can be adjusted interactively by pressing `Calibrate Kernel Size`. This opens a new window in which the user is prompted to adjust the size of a dot (by using a slider of the mouse wheel while hovering over the dot) to the size of the path they're trying to extract. A small extra buffer in size is added. This can be used to adjust the cost image until the desired path becomes clear and well defined.

### Methodology

The project follows a simple methodology somewhat similar to that of the livewire segmentation technique.

* **Grayscaling:** The image is converted to grayscale for simpler processing.
* **Contrasting:** The contrast in the image is enhanced to improve visibility of the path.
* **Smoothing:** Gaussian blurring is applied to improve generalization.
* **Edge detection:** The Canny edge detector is applied to generate an image highlighting the edges in the original image.
* **Convolution:** A disk-shaped kernel, with values increasing radially from the center to the circumference, is convolved with the image to create a new image highlighting the centers of the paths.
* **Cost image creation:** A cost image is generated by first identifying the maximum intensity value of the convolved image, `I_max`. The cost at each pixel is then computed as `(I_max - I(x,y))**4`, where `I(x,y)` represents the intensity at pixel `(x,y)`.
* **Graph construction:** A graph is then constructed by connecting every pixel to it's 8 neighbouring pixels. The cost of going between two given pixels will simply be the average of the two pixel's values in the cost image.
* **Pathfinding:** Djikstra's algorithm is then applied to find the cheapest path from the user placed start point to the user placed end point.

### Known bugs & issues

* Guide points need to be placed in order from start to finish along the path for the extracted path not to overlap incorrectly.
* Dots don't currently scale with the image size, meaning the dots will appear small on large images.
* Exported points may have small inaccuracies in labelling which excact pixel point is the user placed point but is accurate to a few pixels in the image.
    * This is due to the way checking of user placed points is calculated. It is not locked to a specific pixel but is the shortest euclidean distance of the actual point and closest pixel in the path.

### Authors and acknowledgment

This project is developed by Aske T. Rove, Christian L. Bjerregaard & Mikkel W. Breinstrup under the supervision of Hans M. Kjer & Felipe D. Matos at DTU Compute, Department of Applied Mathematics and Computer Science. 

### License

This project is licensed under the MIT License.